@@ -1,6 +1,6 @@
 /* ----------------------------------------------------------------------- *
  *   
- *   Copyright 2007-2009 The NASM Authors - All Rights Reserved
+ *   Copyright 2007-2016 The NASM Authors - All Rights Reserved
  *   See the file AUTHORS included with the NASM distribution for
  *   the specific copyright holders.
  *
@@ -43,6 +43,14 @@
 #ifndef NASM_COMPILER_H
 #define NASM_COMPILER_H 1
 
+#ifdef __DJGPP__
+/* DJGPP has header file problems if __STRICT_ANSI__ is defined */
+# undef __STRICT_ANSI__
+#endif
+
+/* On Microsoft platforms we support multibyte character sets in filenames */
+#define _MBCS 1
+
 #ifdef HAVE_CONFIG_H
 # include "config.h"
 /* autoconf doesn't define these if they are redundant, but we want to
@@ -93,13 +101,17 @@ int snprintf(char *, size_t, const char *, ...);
 #endif
 
 #ifndef HAVE_VSNPRINTF
-# ifdef HAVE__VSNPRINT
+# ifdef HAVE__VSNPRINTF
 #  define vsnprintf _vsnprintf
 # else
 int vsnprintf(char *, size_t, const char *, va_list);
 # endif
 #endif
 
+#if !defined(HAVE_STRLCPY) || !HAVE_DECL_STRLCPY
+size_t strlcpy(char *, const char *, size_t);
+#endif
+
 #ifndef __cplusplus		/* C++ has false, true, bool as keywords */
 # if defined(HAVE_STDBOOL_H) && defined(HAVE_WORKING_BOOL)
 #  include <stdbool.h>
@@ -179,4 +191,13 @@ char *strsep(char **, const char *);
 # define no_return void
 #endif
 
+/*
+ * How to tell the compiler that a function takes a printf-like string
+ */
+#ifdef __GNUC__
+# define printf_func(fmt, list) __attribute__((format(printf, fmt, list)))
+#else
+# define printf_func(fmt, list)
+#endif
+
 #endif	/* NASM_COMPILER_H */
