@@ -1,6 +1,6 @@
 /* ----------------------------------------------------------------------- *
  *   
- *   Copyright 1996-2012 The NASM Authors - All Rights Reserved
+ *   Copyright 1996-2016 The NASM Authors - All Rights Reserved
  *   See the file AUTHORS included with the NASM distribution for
  *   the specific copyright holders.
  *
@@ -41,6 +41,7 @@
 #include <stdlib.h>
 #include <string.h>
 #include <ctype.h>
+#include <errno.h>
 #include <inttypes.h>
 
 #include "nasm.h"
@@ -50,10 +51,6 @@
 int globalbits = 0;    /* defined in nasm.h, works better here for ASM+DISASM */
 static vefunc nasm_verror;	/* Global error handling function */
 
-#ifdef LOGALLOC
-static FILE *logfp;
-#endif
-
 /* Uninitialized -> all zero by C spec */
 const uint8_t zero_buffer[ZERO_BUF_SIZE];
 
@@ -86,93 +83,51 @@ void nasm_error(int severity, const char *fmt, ...)
     va_end(ap);
 }
 
-void nasm_init_malloc_error(void)
+no_return nasm_panic(int flags, const char *fmt, ...)
 {
-#ifdef LOGALLOC
-    logfp = fopen("malloc.log", "w");
-    if (logfp) {
-        setvbuf(logfp, NULL, _IOLBF, BUFSIZ);
-    } else {
-        nasm_error(ERR_NONFATAL | ERR_NOFILE, "Unable to open %s", logfp);
-        logfp = stderr;
-    }
-    fprintf(logfp, "null pointer is %p\n", NULL);
-#endif
+    va_list ap;
+
+    va_start(ap, fmt);
+    nasm_verror(flags | ERR_PANIC, fmt, ap);
+    abort();			/* We should never get here */
+}
+
+no_return nasm_panic_from_macro(const char *file, int line)
+{
+    nasm_panic(ERR_NOFILE, "Internal error at %s:%d\n", file, line);
 }
 
-#ifdef LOGALLOC
-void *nasm_malloc_log(const char *file, int line, size_t size)
-#else
 void *nasm_malloc(size_t size)
-#endif
 {
     void *p = malloc(size);
     if (!p)
         nasm_error(ERR_FATAL | ERR_NOFILE, "out of memory");
-#ifdef LOGALLOC
-    else
-        fprintf(logfp, "%s %d malloc(%ld) returns %p\n",
-                file, line, (long)size, p);
-#endif
     return p;
 }
 
-#ifdef LOGALLOC
-void *nasm_zalloc_log(const char *file, int line, size_t size)
-#else
 void *nasm_zalloc(size_t size)
-#endif
 {
     void *p = calloc(size, 1);
     if (!p)
         nasm_error(ERR_FATAL | ERR_NOFILE, "out of memory");
-#ifdef LOGALLOC
-    else
-        fprintf(logfp, "%s %d calloc(%ld, 1) returns %p\n",
-                file, line, (long)size, p);
-#endif
     return p;
 }
 
-#ifdef LOGALLOC
-void *nasm_realloc_log(const char *file, int line, void *q, size_t size)
-#else
 void *nasm_realloc(void *q, size_t size)
-#endif
 {
     void *p = q ? realloc(q, size) : malloc(size);
     if (!p)
         nasm_error(ERR_FATAL | ERR_NOFILE, "out of memory");
-#ifdef LOGALLOC
-    else if (q)
-        fprintf(logfp, "%s %d realloc(%p,%ld) returns %p\n",
-                file, line, q, (long)size, p);
-    else
-        fprintf(logfp, "%s %d malloc(%ld) returns %p\n",
-                file, line, (long)size, p);
-#endif
     return p;
 }
 
-#ifdef LOGALLOC
-void nasm_free_log(const char *file, int line, void *q)
-#else
 void nasm_free(void *q)
-#endif
 {
-    if (q) {
-#ifdef LOGALLOC
-        fprintf(logfp, "%s %d free(%p)\n", file, line, q);
-#endif
+    if (q)
         free(q);
-    }
 }
 
-#ifdef LOGALLOC
-char *nasm_strdup_log(const char *file, int line, const char *s)
-#else
 char *nasm_strdup(const char *s)
-#endif
 {
     char *p;
     int size = strlen(s) + 1;
@@ -180,20 +135,11 @@ char *nasm_strdup(const char *s)
     p = malloc(size);
     if (!p)
         nasm_error(ERR_FATAL | ERR_NOFILE, "out of memory");
-#ifdef LOGALLOC
-    else
-        fprintf(logfp, "%s %d strdup(%ld) returns %p\n",
-                file, line, (long)size, p);
-#endif
     strcpy(p, s);
     return p;
 }
 
-#ifdef LOGALLOC
-char *nasm_strndup_log(const char *file, int line, const char *s, size_t len)
-#else
 char *nasm_strndup(const char *s, size_t len)
-#endif
 {
     char *p;
     int size = len + 1;
@@ -201,11 +147,6 @@ char *nasm_strndup(const char *s, size_t len)
     p = malloc(size);
     if (!p)
         nasm_error(ERR_FATAL | ERR_NOFILE, "out of memory");
-#ifdef LOGALLOC
-    else
-        fprintf(logfp, "%s %d strndup(%ld) returns %p\n",
-                file, line, (long)size, p);
-#endif
     strncpy(p, s, len);
     p[len] = '\0';
     return p;
@@ -217,6 +158,13 @@ no_return nasm_assert_failed(const char *file, int line, const char *msg)
     exit(1);
 }
 
+void nasm_write(const void *ptr, size_t size, FILE *f)
+{
+    size_t n = fwrite(ptr, 1, size, f);
+    if (n != size)
+        nasm_error(ERR_FATAL, "unable to write output: %s", strerror(errno));
+}
+
 #ifndef nasm_stricmp
 int nasm_stricmp(const char *s1, const char *s2)
 {
@@ -443,38 +391,36 @@ int64_t readstrnum(char *str, int length, bool *warn)
     return charconst;
 }
 
-static int32_t next_seg;
-
-void seg_init(void)
-{
-    next_seg = 0;
-}
-
 int32_t seg_alloc(void)
 {
-    return (next_seg += 2) - 2;
+    static int32_t next_seg = 0;
+    int32_t this_seg = next_seg;
+
+    next_seg += 2;
+
+    return this_seg;
 }
 
 #ifdef WORDS_LITTLEENDIAN
 
 void fwriteint16_t(uint16_t data, FILE * fp)
 {
-    fwrite(&data, 1, 2, fp);
+    nasm_write(&data, 2, fp);
 }
 
 void fwriteint32_t(uint32_t data, FILE * fp)
 {
-    fwrite(&data, 1, 4, fp);
+    nasm_write(&data, 4, fp);
 }
 
 void fwriteint64_t(uint64_t data, FILE * fp)
 {
-    fwrite(&data, 1, 8, fp);
+    nasm_write(&data, 8, fp);
 }
 
 void fwriteaddr(uint64_t data, int size, FILE * fp)
 {
-    fwrite(&data, 1, size, fp);
+    nasm_write(&data, size, fp);
 }
 
 #else /* not WORDS_LITTLEENDIAN */
@@ -483,50 +429,42 @@ void fwriteint16_t(uint16_t data, FILE * fp)
 {
     char buffer[2], *p = buffer;
     WRITESHORT(p, data);
-    fwrite(buffer, 1, 2, fp);
+    nasm_write(buffer, 2, fp);
 }
 
 void fwriteint32_t(uint32_t data, FILE * fp)
 {
     char buffer[4], *p = buffer;
     WRITELONG(p, data);
-    fwrite(buffer, 1, 4, fp);
+    nasm_write(buffer, 4, fp);
 }
 
 void fwriteint64_t(uint64_t data, FILE * fp)
 {
     char buffer[8], *p = buffer;
     WRITEDLONG(p, data);
-    fwrite(buffer, 1, 8, fp);
+    nasm_write(buffer, 8, fp);
 }
 
 void fwriteaddr(uint64_t data, int size, FILE * fp)
 {
     char buffer[8], *p = buffer;
     WRITEADDR(p, data, size);
-    fwrite(buffer, 1, size, fp);
+    nasm_write(buffer, size, fp);
 }
 
 #endif
 
-size_t fwritezero(size_t bytes, FILE *fp)
+void fwritezero(size_t bytes, FILE *fp)
 {
-    size_t count = 0;
     size_t blksize;
-    size_t rv;
 
     while (bytes) {
 	blksize = (bytes < ZERO_BUF_SIZE) ? bytes : ZERO_BUF_SIZE;
 
-	rv = fwrite(zero_buffer, 1, blksize, fp);
-	if (!rv)
-	    break;
-
-	count += rv;
-	bytes -= rv;
+	nasm_write(zero_buffer, blksize, fp);
+	bytes -= blksize;
     }
-
-    return count;
 }
 
 void standard_extension(char *inname, char *outname, char *extension)
@@ -564,7 +502,7 @@ void standard_extension(char *inname, char *outname, char *extension)
 static const char *prefix_names[] = {
     "a16", "a32", "a64", "asp", "lock", "o16", "o32", "o64", "osp",
     "rep", "repe", "repne", "repnz", "repz", "times", "wait",
-    "xacquire", "xrelease"
+    "xacquire", "xrelease", "bnd"
 };
 
 const char *prefix_name(int token)
@@ -790,6 +728,8 @@ int idata_bytes(int opcode)
         return 16;
     case I_DY:
         return 32;
+    case I_DZ:
+        return 64;
     case I_none:
         return -1;
     default:
