@@ -1,6 +1,6 @@
 /* ----------------------------------------------------------------------- *
  *   
- *   Copyright 1996-2009 The NASM Authors - All Rights Reserved
+ *   Copyright 1996-2013 The NASM Authors - All Rights Reserved
  *   See the file AUTHORS included with the NASM distribution for
  *   the specific copyright holders.
  *
@@ -330,6 +330,8 @@ static void as86_out(int32_t segto, const void *data,
         return;
     }
 
+    memset(mydata, 0, sizeof(mydata));
+
     if (type == OUT_RESERVE) {
         if (s) {
             nasm_error(ERR_WARNING, "uninitialized space declared in"
@@ -345,19 +347,20 @@ static void as86_out(int32_t segto, const void *data,
         as86_sect_write(s, data, size);
         as86_add_piece(s, 0, 0L, 0L, size, 0);
     } else if (type == OUT_ADDRESS) {
+        int asize = abs((int)size);
         if (segment != NO_SEG) {
             if (segment % 2) {
                 nasm_error(ERR_NONFATAL, "as86 format does not support"
                       " segment base references");
             } else {
                 offset = *(int64_t *)data;
-                as86_add_piece(s, 1, offset, segment, size, 0);
+                as86_add_piece(s, 1, offset, segment, asize, 0);
             }
         } else {
             p = mydata;
             WRITELONG(p, *(int64_t *)data);
-            as86_sect_write(s, data, size);
-            as86_add_piece(s, 0, 0L, 0L, size, 0);
+            as86_sect_write(s, data, asize);
+            as86_add_piece(s, 0, 0L, 0L, asize, 0);
         }
     } else if (type == OUT_REL2ADR) {
         if (segment == segto)
@@ -544,7 +547,7 @@ static void as86_write_section(struct Section *sect, int index)
                 int32_t tmplen = (length > 64 ? 64 : length);
                 fputc(0x40 | (tmplen & 0x3F), ofile);
                 saa_rnbytes(sect->data, buf, tmplen);
-                fwrite(buf, 1, tmplen, ofile);
+                nasm_write(buf, tmplen, ofile);
                 length -= tmplen;
             } while (length > 0);
             break;
@@ -630,6 +633,7 @@ struct ofmt of_as86 = {
     "Linux as86 (bin86 version 0.3) object files",
     "as86",
     0,
+    32,
     null_debug_arr,
     &null_debug_form,
     as86_stdmac,
@@ -638,6 +642,7 @@ struct ofmt of_as86 = {
     as86_out,
     as86_deflabel,
     as86_section_names,
+    null_sectalign,
     as86_segbase,
     null_directive,
     as86_filename,
