@@ -1,6 +1,6 @@
 /* ----------------------------------------------------------------------- *
  *   
- *   Copyright 1996-2009 The NASM Authors - All Rights Reserved
+ *   Copyright 1996-2013 The NASM Authors - All Rights Reserved
  *   See the file AUTHORS included with the NASM distribution for
  *   the specific copyright holders.
  *
@@ -130,7 +130,7 @@ static struct Section {
 
     struct bin_label *labels;   /* linked-list of label handles for map output. */
     struct bin_label **labels_end;      /* Holds address of end of labels list. */
-    struct Section *ifollows;   /* Points to previous section (implicit follows). */
+    struct Section *prev;       /* Points to previous section (implicit follows). */
     struct Section *next;       /* This links sections with a defined start address. */
 
 /* The extended bin format allows for sections to have a "virtual"
@@ -201,28 +201,22 @@ static struct Section *find_section_by_index(int32_t index)
 }
 
 static struct Section *create_section(char *name)
-{                               /* Create a new section. */
-    last_section->next = nasm_malloc(sizeof(struct Section));
-    last_section->next->ifollows = last_section;
-    last_section = last_section->next;
-    last_section->labels = NULL;
-    last_section->labels_end = &(last_section->labels);
+{
+    struct Section *s = nasm_zalloc(sizeof(*s));
 
-    /* Initialize section attributes. */
-    last_section->name = nasm_strdup(name);
-    last_section->contents = saa_init(1L);
-    last_section->follows = last_section->vfollows = 0;
-    last_section->length = 0;
-    last_section->flags = 0;
-    last_section->align = 0;
-    last_section->valign = 0;
-    last_section->start = 0;
-    last_section->vstart = 0;
-    last_section->next = NULL;
+    s->prev         = last_section;
+    s->name         = nasm_strdup(name);
+    s->labels_end   = &(s->labels);
+    s->contents     = saa_init(1L);
 
     /* Register our sections with NASM. */
-    last_section->vstart_index = seg_alloc();
-    last_section->start_index = seg_alloc();
+    s->vstart_index = seg_alloc();
+    s->start_index  = seg_alloc();
+
+    /* FIXME: Append to a tail, we need some helper */
+    last_section->next = s;
+    last_section = s;
+
     return last_section;
 }
 
@@ -489,9 +483,9 @@ static void bin_cleanup(int debuginfo)
                     nasm_error(ERR_FATAL|ERR_NOFILE,
                           "section %s vfollows unknown section (%s)",
                           g->name, g->vfollows);
-            } else if (g->ifollows != NULL)
-                for (s = sections; s && (s != g->ifollows); s = s->next) ;
-            /* The .bss section is the only one with ifollows = NULL.
+            } else if (g->prev != NULL)
+                for (s = sections; s && (s != g->prev); s = s->next) ;
+            /* The .bss section is the only one with prev = NULL.
 	       In this case we implicitly follow the last progbits
 	       section.  */
             else
@@ -548,6 +542,8 @@ static void bin_cleanup(int debuginfo)
 
         nasm_assert(r->bytes <= 8);
 
+        memset(mydata, 0, sizeof(mydata));
+
         saa_fread(r->target->contents, r->posn, mydata, r->bytes);
         p = mydata;
         l = 0;
@@ -768,6 +764,9 @@ static void bin_out(int32_t segto, const void *data,
 
     switch (type) {
     case OUT_ADDRESS:
+    {
+        int asize = abs((int)size);
+
         if (segment != NO_SEG && !find_section_by_index(segment)) {
             if (segment % 2)
                 nasm_error(ERR_NONFATAL, "binary output format does not support"
@@ -779,12 +778,19 @@ static void bin_out(int32_t segto, const void *data,
         }
         if (s->flags & TYPE_PROGBITS) {
             if (segment != NO_SEG)
-                add_reloc(s, size, segment, -1L);
+                add_reloc(s, asize, segment, -1L);
             p = mydata;
-	    WRITEADDR(p, *(int64_t *)data, size);
-            saa_wbytes(s->contents, mydata, size);
+	    WRITEADDR(p, *(int64_t *)data, asize);
+            saa_wbytes(s->contents, mydata, asize);
         }
+
+        /*
+         * Reassign size with sign dropped, we will need it
+         * for section length calculation.
+         */
+        size = asize;
 	break;
+    }
 
     case OUT_RAWDATA:
         if (s->flags & TYPE_PROGBITS)
@@ -1262,7 +1268,7 @@ static int32_t bin_secname(char *name, int pass, int *bits)
             sec->flags |= TYPE_DEFINED | TYPE_PROGBITS;
         else if (!strcmp(name, ".bss")) {
             sec->flags |= TYPE_DEFINED | TYPE_NOBITS;
-            sec->ifollows = NULL;
+            sec->prev = NULL;
         }
     }
 
@@ -1425,7 +1431,6 @@ static void srec_init(void)
 
 static void binfmt_init(void)
 {
-    maxbits = 64;               /* Support 64-bit Segments */
     relocs = NULL;
     reloctail = &relocs;
     origin_defined = 0;
@@ -1433,18 +1438,13 @@ static void binfmt_init(void)
     nsl_tail = &no_seg_labels;
 
     /* Create default section (.text). */
-    sections = last_section = nasm_malloc(sizeof(struct Section));
-    last_section->next = NULL;
-    last_section->name = nasm_strdup(".text");
-    last_section->contents = saa_init(1L);
-    last_section->follows = last_section->vfollows = 0;
-    last_section->ifollows = NULL;
-    last_section->length = 0;
-    last_section->flags = TYPE_DEFINED | TYPE_PROGBITS;
-    last_section->labels = NULL;
-    last_section->labels_end = &(last_section->labels);
-    last_section->start_index = seg_alloc();
-    last_section->vstart_index = seg_alloc();
+    sections = last_section = nasm_zalloc(sizeof(struct Section));
+    last_section->name          = nasm_strdup(".text");
+    last_section->contents      = saa_init(1L);
+    last_section->flags         = TYPE_DEFINED | TYPE_PROGBITS;
+    last_section->labels_end    = &(last_section->labels);
+    last_section->start_index   = seg_alloc();
+    last_section->vstart_index  = seg_alloc();
 }
 
 /* Generate binary file output */
@@ -1475,8 +1475,8 @@ static void do_output_bin(void)
 }
 
 /* Generate Intel hex file output */
-static int write_ith_record(unsigned int len, uint16_t addr,
-			    uint8_t type, void *data)
+static void write_ith_record(unsigned int len, uint16_t addr,
+                             uint8_t type, void *data)
 {
     char buf[1+2+4+2+255*2+2+2];
     char *p = buf;
@@ -1495,10 +1495,7 @@ static int write_ith_record(unsigned int len, uint16_t addr,
 	p += sprintf(p, "%02X", dptr[i]);
     p += sprintf(p, "%02X\n", csum);
 
-    if (fwrite(buf, 1, p-buf, ofile) != (size_t)(p-buf))
-	return -1;
-
-    return 0;
+    nasm_write(buf, p-buf, ofile);
 }
 
 static void do_output_ith(void)
@@ -1549,8 +1546,8 @@ static void do_output_ith(void)
 }
 
 /* Generate Motorola S-records */
-static int write_srecord(unsigned int len,  unsigned int alen,
-			 uint32_t addr, uint8_t type, void *data)
+static void write_srecord(unsigned int len,  unsigned int alen,
+                          uint32_t addr, uint8_t type, void *data)
 {
     char buf[2+2+8+255*2+2+2];
     char *p = buf;
@@ -1583,10 +1580,7 @@ static int write_srecord(unsigned int len,  unsigned int alen,
 	p += sprintf(p, "%02X", dptr[i]);
     p += sprintf(p, "%02X\n", csum);
 
-    if (fwrite(buf, 1, p-buf, ofile) != (size_t)(p-buf))
-	return -1;
-
-    return 0;
+    nasm_write(buf, p-buf, ofile);
 }
 
 static void do_output_srec(void)
@@ -1665,6 +1659,7 @@ struct ofmt of_bin = {
     "flat-form binary files (e.g. DOS .COM, .SYS)",
     "bin",
     0,
+    64,
     null_debug_arr,
     &null_debug_form,
     bin_stdmac,
@@ -1684,6 +1679,7 @@ struct ofmt of_ith = {
     "Intel hex",
     "ith",
     OFMT_TEXT,
+    64,
     null_debug_arr,
     &null_debug_form,
     bin_stdmac,
@@ -1702,7 +1698,8 @@ struct ofmt of_ith = {
 struct ofmt of_srec = {
     "Motorola S-records",
     "srec",
-    0,
+    OFMT_TEXT,
+    64,
     null_debug_arr,
     &null_debug_form,
     bin_stdmac,
