@@ -1,6 +1,6 @@
 /* ----------------------------------------------------------------------- *
  *   
- *   Copyright 1996-2009 The NASM Authors - All Rights Reserved
+ *   Copyright 1996-2013 The NASM Authors - All Rights Reserved
  *   See the file AUTHORS included with the NASM distribution for
  *   the specific copyright holders.
  *
@@ -127,7 +127,10 @@ static void dbg_out(int32_t segto, const void *data,
     int32_t ldata;
     int id;
 
-    fprintf(ofile, "out to %"PRIx32", len = %"PRIu64": ", segto, size);
+    if (type == OUT_ADDRESS)
+        fprintf(ofile, "out to %"PRIx32", len = %d: ", segto, (int)abs((int)size));
+    else
+        fprintf(ofile, "out to %"PRIx32", len = %"PRIu64": ", segto, size);
 
     switch (type) {
     case OUT_RESERVE:
@@ -144,8 +147,12 @@ static void dbg_out(int32_t segto, const void *data,
         break;
     case OUT_ADDRESS:
 	ldata = *(int64_t *)data;
-        fprintf(ofile, "addr %08"PRIx32" (seg %08"PRIx32", wrt %08"PRIx32")\n", ldata,
-                segment, wrt);
+        fprintf(ofile, "addr %08"PRIx32" (seg %08"PRIx32", wrt %08"PRIx32")\n",
+                ldata, segment, wrt);
+        break;
+    case OUT_REL1ADR:
+        fprintf(ofile, "rel1adr %02"PRIx8" (seg %08"PRIx32")\n",
+		(uint8_t)*(int64_t *)data, segment);
         break;
     case OUT_REL2ADR:
         fprintf(ofile, "rel2adr %04"PRIx16" (seg %08"PRIx32")\n",
@@ -166,6 +173,12 @@ static void dbg_out(int32_t segto, const void *data,
     }
 }
 
+static void dbg_sectalign(int32_t seg, unsigned int value)
+{
+    fprintf(ofile, "set alignment (%d) for segment (%u)\n",
+            seg, value);
+}
+
 static int32_t dbg_segbase(int32_t segment)
 {
     return segment;
@@ -193,7 +206,7 @@ static int dbg_set_info(enum geninfo type, char **val)
 char *types[] = {
     "unknown", "label", "byte", "word", "dword", "float", "qword", "tbyte"
 };
-void dbgdbg_init(void)
+static void dbgdbg_init(void)
 {
     fprintf(ofile, "   With debug info\n");
 }
@@ -251,6 +264,7 @@ struct ofmt of_dbg = {
     "Trace of all info passed to output stage",
     "dbg",
     OFMT_TEXT,
+    64,
     debug_debug_arr,
     &debug_debug_form,
     NULL,
@@ -259,6 +273,7 @@ struct ofmt of_dbg = {
     dbg_out,
     dbg_deflabel,
     dbg_section_names,
+    dbg_sectalign,
     dbg_segbase,
     dbg_directive,
     dbg_filename,
