@@ -190,7 +190,6 @@ static int32_t elf_gottpoff_sect;
 
 static void elf_init(void)
 {
-    maxbits = 64;
     sects = NULL;
     nsects = sectlen = 0;
     syms = saa_init((int32_t)sizeof(struct elf_symbol));
@@ -745,7 +744,7 @@ static void elf_out(int32_t segto, const void *data,
     case OUT_ADDRESS:
     {
         int isize = (int)size;
-        int asize = abs(size);
+        int asize = abs((int)size);
 
         addr = *(int64_t *)data;
         if (segment == NO_SEG) {
@@ -983,7 +982,7 @@ static void elf_write(void)
     /*
      * Output the ELF header.
      */
-    fwrite("\177ELF\1\1\1", 7, 1, ofile);
+    nasm_write("\177ELF\1\1\1", 7, ofile);
     fputc(elf_osabi, ofile);
     fputc(elf_abiver, ofile);
     fwritezero(7, ofile);
@@ -1334,7 +1333,7 @@ static void elf_write_sections(void)
             if (elf_sects[i].is_saa)
                 saa_fpwrite(elf_sects[i].data, ofile);
             else
-                fwrite(elf_sects[i].data, len, 1, ofile);
+                nasm_write(elf_sects[i].data, len, ofile);
             fwritezero(align, ofile);
         }
 }
@@ -1416,6 +1415,7 @@ struct ofmt of_elfx32 = {
     "ELFX32 (x86_64) object files (e.g. Linux)",
     "elfx32",
     0,
+    64,
     elfx32_debugs_arr,
     &df_stabs,
     elf_stdmac,
@@ -1641,7 +1641,7 @@ static void stabsx32_generate(void)
          * the source-file, the n_desc field should be set to the number
          * of remaining stabs
          */
-        WRITE_STAB(sptr, fileidx[0], 0, 0, 0, strlen(allfiles[0] + 12));
+        WRITE_STAB(sptr, fileidx[0], 0, 0, 0, stabstrlen);
 
         /* this is the stab for the main source file */
         WRITE_STAB(sptr, fileidx[mainfileindex], N_SO, 0, 0, 0);
