@@ -45,7 +45,7 @@ my($what, $in, $out) = @ARGV;
 #
 open(IN, "< $in") or die "$0: cannot open: $in\n";
 while (defined($line = <IN>)) {
-    chomp $line;
+    $line =~ s/\r?\n$//;	# Remove trailing \r\n or \n
     $line =~ s/^\s+//;		# Remove leading whitespace
     $line =~ s/\s*\#.*$//;	# Remove comments and trailing whitespace
     next if ($line eq '');
@@ -164,7 +164,7 @@ if ($what eq 'c') {
     }
 
     my @hashinfo = gen_perfect_hash(\%tokens);
-    if (!defined(@hashinfo)) {
+    if (!@hashinfo) {
 	die "$0: no hash found\n";
     }
 
@@ -207,7 +207,7 @@ if ($what eq 'c') {
     # Put a large value in unused slots.  This makes it extremely unlikely
     # that any combination that involves unused slot will pass the range test.
     # This speeds up rejection of unrecognized tokens, i.e. identifiers.
-    print OUT "#define UNUSED 16383\n";
+    print OUT "#define UNUSED (65535/3)\n";
 
     print OUT "    static const int16_t hash1[$n] = {\n";
     for ($i = 0; $i < $n; $i++) {
